/*
 * Arc Menu: The new applications menu for Gnome 3.
 *
 * Copyright (C) 2017 Alexander Rüedlinger
 * Copyright (C) 2017-2018 LinxGem33
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Import Libraries
const Lang = imports.lang;
const Shell = imports.gi.Shell;
const St = imports.gi.St;
const Clutter = imports.gi.Clutter;
const Main = imports.ui.main;
const Tweener = imports.ui.tweener;
const PopupMenu = imports.ui.popupMenu;
const GLib = imports.gi.GLib;
const Signals = imports.signals;
const AccountsService = imports.gi.AccountsService;
const Gio = imports.gi.Gio;
//Add kylin code for import meta
const Meta = imports.gi.Meta;
const Util = imports.misc.util;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Gettext = imports.gettext.domain(Me.metadata['gettext-domain']);
//Kylin code for add import
const SecondaryMenu = Me.imports.secondaryMenu;
const _ = Gettext.gettext;

const DND = imports.ui.dnd;
const LoginManager = imports.misc.loginManager;

// Menu Size variables
const APPLICATION_ICON_SIZE = 32;
const TOOLTIP_LABEL_SHOW_TIME = 0.15;
const TOOLTIP_LABEL_HIDE_TIME = 0.1;

//Add kylin code 
const APP_IN_FIRST_AREA_BEGIN = 0;
const APP_IN_FIRST_AREA_RECENT = 1;
const APP_IN_DIR_AREA = 2;
const APP_IN_SEARCH_AREA = 3;
const DESKTOP_INTERFACE_SCHEMA = 'org.gnome.desktop.interface';
const KEY_TEXT_SCALING_FACTOR  = 'text-scaling-factor';
const back_icon_path = Me.path + '/media/back-symbolic.svg';
const go_icon_path = Me.path + '/media/go-symbolic.svg';
const default_user = Me.path + '/media/default-user-icon.png';
const computer_icon_path = Me.path + '/media/computer-symbolic.png';
const home_icon_path = Me.path + '/media/home-symbolic.png';
const documents_icon_path = Me.path + '/media/documents-symbolic.png';
const pictures_icon_path = Me.path + '/media/pictures-symbolic.png';
const music_icon_path = Me.path + '/media/music-symbolic.png';
const videos_icon_path = Me.path + '/media/videos-symbolic.png';
const download_icon_path = Me.path + '/media/download-symbolic.png';
const control_panel_icon_path = Me.path + '/media/control-panel-symbolic.png';
const devices_printers_icon_path = Me.path + '/media/device-printer-symbolic.png';
const default_programs_icon_path = Me.path + '/media/default-pro-symbolic.png';
const help_support_icon_path = Me.path + '/media/help-support-symbolic.png';
const about_kylin_icon_path = Me.path +  '/media/logo.png';

function setIconAsync(icon, gioFile, fallback_icon_name) {
    gioFile.load_contents_async(null, function(source, result) {
        try {
            let bytes = source.load_contents_finish(result)[1];
            icon.gicon = Gio.BytesIcon.new(bytes);
        } catch(err) {
            icon.icon_name = fallback_icon_name;
        }
    });
}

// Removing the default behaviour which selects a hovered item if the space key is pressed.
// This avoids issues when searching for an app with a space character in its name.
var BaseMenuItem = new Lang.Class({
    Name: 'BaseMenuItem',
    Extends: PopupMenu.PopupBaseMenuItem,

    _onButtonPressEvent: function (actor, event) {
        // This is the CSS active state
        let button = event.get_button();
        if(button == 1) {
            this.actor.add_style_pseudo_class ('active');
        }
        return Clutter.EVENT_PROPAGATE;
    },

    _onButtonReleaseEvent: function (actor, event) {
        this.actor.remove_style_pseudo_class ('active');
        let button = event.get_button();
        if(button == 1) {
            this.activate(event);
        }
        return Clutter.EVENT_STOP;
    },

    _onKeyPressEvent: function (actor, event) {
        let symbol = event.get_key_symbol();
        if (symbol == Clutter.KEY_Return ||
            symbol == Clutter.KEY_KP_Enter) {
            this.activate(event);
            return Clutter.EVENT_STOP;
        }
        return Clutter.EVENT_PROPAGATE;
    }
});

// Menu item to launch GNOME activities overview
var ActivitiesMenuItem = new Lang.Class({
    Name: 'ActivitiesMenuItem',
    Extends: BaseMenuItem,

    // Initialize the menu item
    _init: function(button) {
	    this.parent();
        this._button = button;
        this._icon = new St.Icon({
            icon_name: 'view-fullscreen-symbolic',
            style_class: 'popup-menu-icon',
            icon_size: 16
        });
        this.actor.add_child(this._icon);
        let label = new St.Label({
            text: _("Activities Overview"),
            y_expand: true,
            y_align: Clutter.ActorAlign.CENTER
        });
        this.actor.add_child(label);
    },

    // Activate the menu item (Open activities overview)
    activate: function(event) {
        this._button.menu.toggle();
        Main.overview.toggle();
	    this.parent(event);
    }
});

/**
 * A class representing a Tooltip.
 */
var Tooltip = new Lang.Class({
    Name: 'Tooltip',

    _init: function(sourceActor, text) {
        this.sourceActor = sourceActor;
        this.actor = new St.Label({
            style_class: 'tooltip-label',
            text: text,
            opacity: 0
        });
        global.stage.add_actor(this.actor);
        this.actor.show();
        this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
    },

    show: function() {
        let [stageX, stageY] = this.sourceActor.get_transformed_position();
        let [width, height] = this.sourceActor.get_transformed_size();
        let y = stageY - height/1.25;
        let x = stageX - Math.round((this.actor.get_width() - width)/2);

        this.actor.show();
        this.actor.set_position(x, y);
        Tweener.addTween(this.actor, {
            opacity: 255,
            time: TOOLTIP_LABEL_SHOW_TIME,
            transition: 'easeOutQuad'
        });
    },

    hide: function() {
        Tweener.addTween(this.actor, {
            opacity: 0,
            time: TOOLTIP_LABEL_HIDE_TIME,
            transition: 'easeOutQuad',
            onComplete: Lang.bind(this, function() {
                this.actor.hide();
            })
        });
    },

    _onDestroy: function() {
        global.stage.remove_actor(this.actor);
    }
});

/**
 * A base class for custom session buttons.
 */
var SessionButton = new Lang.Class({
    Name: 'SessionButton',

    _init: function(button, accessible_name, icon_name) {
        this._button = button;
        this.actor = new St.Button({
            reactive: true,
            can_focus: true,
            track_hover: true,
            accessible_name: accessible_name,
            style_class: 'system-menu-action'
        });
	this._useTooltips = true;
        this.tooltip = new Tooltip(this.actor, accessible_name);
        this.tooltip.hide();
        this.actor.child = new St.Icon({ icon_name: icon_name });
        this.actor.connect('clicked', Lang.bind(this, this._onClick));
        this.actor.connect('notify::hover', Lang.bind(this, this._onHover));
    },

       useTooltips: function(useTooltips) {
       this._useTooltips = useTooltips;
    },
	
    _onClick: function() {
        this._button.menu.toggle();
        this.activate();
    },

    activate: function() {
        // Button specific action
    },

    _onHover: function() {
        // TODO: implement tooltips
	          if (!this._useTooltips)
           return;

        if (this.actor.hover) { // mouse pointer hovers over the button
            this.tooltip.show();
        } else { // mouse pointer leaves the button area
            this.tooltip.hide();
        }
    }
});

// Power Button
var PowerButton = new Lang.Class({
    Name: 'PowerButton',
    Extends: BaseMenuItem, //Add kylin code

    // Initialize the button
    _init: function(button) {
	//Add kylin code
        this.parent();
        this.actor.remove_all_children();
        this._button = button;
        let label = new St.Label({
            text: _("Power Off"),
            y_expand: true,
            style_class: 'app-well-menu'
        });
        this._tipText = _("Closes all open programs, shuts down Windows, and then turns off your computer.")
        this.actor.add_actor(label);
        label.add_style_class_name('power-button-label-text');
        this.actor.add_style_class_name('app-well-menu');
        this.actor.add_style_class_name('power-button-label');
        this.actor.connect('enter-event', Lang.bind(this, this._onMouseEnter));
        this.actor.connect('leave-event', Lang.bind(this, this._offMouseEnter));
    },
  
    //Add kylin code
    _onMouseEnter: function(actor, event) {
//        let [x,y]= event.get_coords();
//        this._button.tipLabelShow(this._tipText, x, y);
    },

    _offMouseEnter: function(actor, event) {
//        this._button.tipLabelHiddle();
    },

    // Activate the button (Shutdown)
    activate: function() {
	//Add kylin code
//	this._button.tipLabelHiddle();
        this._button.menu.toggle();
        this._button._session.ShutdownRemote(0);
    }
});

//Add kylin code for power menu
var PowerMenuButton = new Lang.Class({
    Name: 'PowerMenuButton',
    Extends: PopupMenu.PopupBaseMenuItem,

    _init: function(button) {
        this.parent();
        this._button = button;
        this.actor.add_style_class_name('power-menu-b');

        this._isPopup = 0;
        this._draggable = DND.makeDraggable(this.actor);
        this.isDraggableApp = true;
        this._menuManager = new PopupMenu.PopupMenuManager(this);
        this._draggable.connect('drag-begin', Lang.bind(this, this._onDragBegin));
        this._draggable.connect('drag-cancelled', Lang.bind(this, this._onDragCancelled));
        this._draggable.connect('drag-end', Lang.bind(this, this._onDragEnd));
        this.actor.connect('enter-event', Lang.bind(this, this._onMouseEnter));
        this.actor.connect('leave-event', Lang.bind(this, this._offMouseEnter));
    },

    _onMouseEnter: function(actor, event) {
        this._isPopup = 1;
    },

    _offMouseEnter: function(actor, event) {
    },

    _onDragBegin: function() {
        Main.overview.beginItemDrag(this);
    },

    _onDragCancelled: function() {
        Main.overview.cancelledItemDrag(this);
    },

    _onDragEnd: function() {
        Main.overview.endItemDrag(this);
    },
    _onMenuPoppedDown: function() {
        this.actor.sync_hover();
        this.emit('menu-state-changed', false);
    },

   _onButtonPressEvent: function(actor, event) {
        let button = event.get_button();
        if (button == 3) {

        } else if (button == 1) {
            let [x,y]= this.actor.get_transformed_position();
            this.popupMenu(x, y);
            return Clutter.EVENT_STOP;
        }
        return Clutter.EVENT_PROPAGATE;
    },

    _onButtonReleaseEvent: function (actor, event) {
        return Clutter.EVENT_STOP;
    },

    popupMenu: function(x, y) {
        if (this._draggable)
            this._draggable.fakeRelease();

        if (!this._menu) {
            this._menu = new SecondaryMenu.PowerItemMenu(this, this._button);
            this._menu.actor.add_style_class_name('power-menu');
            this._menu.connect('activate-window', Lang.bind(this, function (menu, window) {
                this.activateWindow(window);
            }));
            this._menu.connect('open-state-changed', Lang.bind(this, function (menu, isPoppedUp) {
                if (!isPoppedUp)
                    this._onMenuPoppedDown();
            }));
            let id = Main.overview.connect('hiding', Lang.bind(this, function () { this._menu.close(); }));
            this.actor.connect('destroy', function() {
                Main.overview.disconnect(id);
            });
            this._menuManager.addMenu(this._menu);
        }

        this.emit('menu-state-changed', true);
	this._menu.popup();
        this._menuManager.ignoreRelease();
        return false;
    }
});

// Logout Button
var LogoutButton = new Lang.Class({
    Name: 'LogoutButton',
    Extends: SessionButton,

    // Initialize the button
    _init: function(button) {
        this.parent(button, _("Log Out"), 'application-exit-symbolic');
    },

    // Activate the button (Logout)
    activate: function() {
        this._button._session.LogoutRemote(0);
    }
});

// Suspend Button
var SuspendButton = new Lang.Class({
    Name: 'SuspendButton',
    Extends: SessionButton,

    // Initialize the button
    _init: function(button) {
        this.parent(button, _("Suspend"), 'media-playback-pause-symbolic');
    },

    // Activate the button (Suspend the system)
    activate: function() {
        let loginManager = LoginManager.getLoginManager();
        loginManager.canSuspend(Lang.bind(this, function(result) {
            if (result) {
                loginManager.suspend();
            }
        }));
    }
});

// Lock Screen Button
var LockButton = new Lang.Class({
    Name: 'LockButton',
    Extends: SessionButton,

    // Initialize the button
    _init: function(button) {
        this.parent(button, _("Lock"), 'changes-prevent-symbolic');
    },

    // Activate the button (Lock the screen)
    activate: function() {
        Main.screenShield.lock(true);
    }
});

// Menu item to go back to category view
var BackMenuItem = new Lang.Class({
    Name: 'BackMenuItem',
    Extends: BaseMenuItem,

    // Initialize the button
    _init: function(button) {
        this.parent();
        this._button = button;

        //Add kylin code for user kylin style class
        this._tipText = _("Return to the previous view.");
        this.actor.remove_all_children();
        this.actor.add_style_class_name('app-menu-item');
        this._icon = new St.Icon({
            style_class: 'popup-menu-icon',
        });
        this._icon.add_style_class_name('dir-oper-icon');
        let backLabel = new St.Label({
            style_class: 'back-label',
            text: _("Back"),
            y_expand: true,
            y_align: Clutter.ActorAlign.CENTER
        });
        if (GLib.file_test(back_icon_path, GLib.FileTest.EXISTS)) {
            this._icon.set_gicon(Gio.icon_new_for_string(back_icon_path));
        }
        this.actor.add_child(this._icon);
        this.actor.add_child(backLabel);
        this.actor.connect('enter-event', Lang.bind(this, this._onMouseEnter));
        this.actor.connect('leave-event', Lang.bind(this, this._offMouseEnter));
    },

    //Add kylin code
    _onMouseEnter: function(actor, event) {
//        let [x,y]= event.get_coords();
//        this._button.tipLabelShow(this._tipText, x, y);
    },

    _offMouseEnter: function(actor, event) {
//        this._button.tipLabelHiddle();
    },

    // Activate the button (go back to category view)
    activate: function(event) {
        this._button.selectCategory(null);
        if (this._button.searchActive) this._button.resetSearch();
	    this.parent(event);
    }
});

//Add kylin code for add show all program button
var GoMenuItem = new Lang.Class({
    Name: 'GoMenuItem',
    Extends: BaseMenuItem,

    // Initialize the button
    _init: function(button) {
        this.parent();
        this.actor.remove_all_children();
        this.actor.add_style_class_name('app-menu-item');
        this._button = button;
        this._tipText = _("Open your program list.");
        this._icon = new St.Icon({
            style_class: 'popup-menu-icon',
        });
        this._icon.add_style_class_name('dir-oper-icon');
        let goLabel = new St.Label({
            style_class: 'back-label',
            text: _("All Programs"),
            y_expand: true,
            y_align: Clutter.ActorAlign.CENTER
        });
        if (GLib.file_test(go_icon_path, GLib.FileTest.EXISTS)) {
            this._icon.set_gicon(Gio.icon_new_for_string(go_icon_path));
        }
        this.actor.add_child(this._icon);
        this.actor.add_child(goLabel);
        this.actor.connect('enter-event', Lang.bind(this, this._onMouseEnter));
        this.actor.connect('leave-event', Lang.bind(this, this._offMouseEnter));
    },
    _onMouseEnter: function(actor, event) {
//        let [x,y]= event.get_coords();
//        this._button.tipLabelShow(this._tipText, x, y);
    },

    _offMouseEnter: function(actor, event) {
//        this._button.tipLabelHiddle();
    },

    // Activate the button (go back to category view)
    activate: function(event) {
//        this._button.tipLabelHiddle();
        this._button._clearApplicationsBox();
        this._button._loadCategories();
        this._button.goButton.actor.hide();
        this._button.backButton.actor.show();
        this.parent(event);
    }
});

// Menu shortcut item class
var ShortcutMenuItem = new Lang.Class({
    Name: 'ShortcutMenuItem',
    Extends: BaseMenuItem,

    // Initialize the menu item
    _init: function(button, name, icon, command) {
        this.parent();
        this._button = button;
        this._command = command;
	this._name = name; //Add kylin code
	this.actor.remove_all_children(); //Add kylin code to clear all children
        this.actor.add_style_class_name('place-menu-item'); //Add kylin code for use kylin style
	this._icon = new St.Icon({style_class: 'user-icon'});
        let label = new St.Label({ 
		text: name, 
		y_expand: true,
                style_class: 'place-menu-item-label',
                y_align: Clutter.ActorAlign.CENTER });
        this.actor.add_child(label, { expand: true });
	//Add kylin code for mouse event
	this._setTipTextAndIcon();
	this.actor.connect('enter-event', Lang.bind(this, this._onMouseEnter));
        this.actor.connect('leave-event', Lang.bind(this, this._offMouseEnter));
    },

    // Activate the menu item (Launch the shortcut)
    activate: function(event) {
        Util.spawnCommandLine(this._command);
	//Add kylin code
	this._button.user.swapIcon(this._button.user._userIcon);
        this._button.menu.toggle();
//        this._button.tipLabelHiddle();
	this.parent(event);
    },

    // Add kylin code for mouse in
    _onMouseEnter: function(actor, event) {
        this._button.user.swapIcon(this._icon);
//        let [x,y]= event.get_coords();
//        this._button.tipLabelShow(this._tipText, x, y);
    },

    _offMouseEnter: function(actor, event) {
        this._button.user.swapIcon(this._button.user._userIcon);
//        this._button.tipLabelHiddle();
    },

    //Add kylin code for set tip text and icon
    _setTipTextAndIcon: function() {
        this._tipText = "";
        let iconPath;

        if(this._name == _("Control Panel")) {
            iconPath = control_panel_icon_path;
            this._tipText = _("Change  settings and customize the functionality of your computer.");
        }
        else if(this._name == _("Devices and Printers")) {
            iconPath = devices_printers_icon_path;
            this._tipText = _("View  and manage devices, printers, and print jobs.");
        }
        else if(this._name == _("Default Programs")) {
            iconPath = default_programs_icon_path;
            this._tipText = _("Choose default programs for web browsing, e-mail, playing  music, and other activites.");
        }
        else if(this._name == _("Help and Support")) {
            iconPath = help_support_icon_path;
            this._tipText = _("Find Help topics, tutorials,troublesshooting, and other support services.");
        }
	else if(this._name == _("About Kylin")) {
            iconPath = about_kylin_icon_path;
            this._tipText = _("View system information.");
        }

        if (GLib.file_test(iconPath, GLib.FileTest.EXISTS)) {
            this._icon.set_gicon(Gio.icon_new_for_string(iconPath));
        }
    }
});

// Menu item which displays the current user
var UserMenuItem = new Lang.Class({
    Name: 'UserMenuItem',
    Extends: BaseMenuItem,

    // Initialize the menu item
    _init: function(button) {
        this.parent();
        this._button = button;
	this.actor.remove_all_children(); //Add kylin code to clear all children
        let username = GLib.get_user_name();
        this._user = AccountsService.UserManager.get_default().get_user(username);
        this._userIcon = new St.Icon({
            style_class: 'user-icon', //Add kylin code
        });
	this.actor.set_style_class_name('user-menu-item'); //Add kylin code
        this.actor.add_child(this._userIcon);
        this._userLoadedId = this._user.connect('notify::is_loaded', Lang.bind(this, this._onUserChanged));
        this._userChangedId = this._user.connect('changed', Lang.bind(this, this._onUserChanged));
        this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
        this._onUserChanged();
	//Add kylin code for mouse move
	this.actor.connect('enter-event', Lang.bind(this, this._onMouseEnter));
        this.actor.connect('leave-event', Lang.bind(this, this._offMouseEnter));
        if (GLib.file_test(default_user, GLib.FileTest.EXISTS))
            this._userIcon.set_gicon(Gio.icon_new_for_string(default_user));
    },

    // Activate the menu item (Open user account settings)
    activate: function(event) {
        global.screen.set_cursor(Meta.Cursor.DEFAULT);
        Util.spawnCommandLine("gnome-control-center user-accounts");
        this._button.menu.toggle();
	    this.parent(event);
    },

    // Handle changes to user information (redisplay new info)
    _onUserChanged: function() {
	//Add kylin code
       	if (this._user.is_loaded) {
            if (this._userIcon) {
                let iconFileName = this._user.get_icon_file();
                if(iconFileName.indexOf(".face") == -1 ) {
                    let iconFile = Gio.file_new_for_path(iconFileName);
                    setIconAsync(this._userIcon, iconFile, 'avatar-default');
                } else {
                    if (GLib.file_test(default_user, GLib.FileTest.EXISTS)) {
                        this._userIcon.set_gicon(Gio.icon_new_for_string(default_user));
                    }
                }
            }
        }
    },
  
    //Add kylin code for mouse in 
    _onMouseEnter: function(actor, event) {
        global.screen.set_cursor(Meta.Cursor.POINTING_HAND);
    },

   //Add kylin code for mouse out
    _offMouseEnter: function(actor, event) {
        global.screen.set_cursor(Meta.Cursor.DEFAULT);
    },

    //Add kylin code for hide icon
    _hideIcon: function(icon) {
        this.icon = icon;
        Tweener.addTween(this.actor,
                     { opacity: 50,
                       time: 0.5,
                       delay: 0.25,
                       transition: 'easeOutQuad',
                       onComplete: Lang.bind(this, this._showIcon)
                     });

    },

    //Add kylin code for show icon
    _showIcon: function() {
        this.actor.opacity = 155;
        this.actor.remove_all_children();
        this.actor.add_child(this.icon);
        Tweener.removeTweens(this.actor);
        Tweener.addTween(this.actor,
                     { opacity: 255,
                       time: 0.25,
                       transition: 'easeInQuart'
                     });
    },

    //Add kylin code for swap icon
    swapIcon: function(icon) {
        Tweener.removeTweens(this.actor);
        this._hideIcon(icon);
    },

    // Destroy the menu item
    _onDestroy: function() {
        if (this._userLoadedId != 0) {
            this._user.disconnect(this._userLoadedId);
            this._userLoadedId = 0;
        }
        if (this._userChangedId != 0) {
            this._user.disconnect(this._userChangedId);
            this._userChangedId = 0;
        }
    }
});

// Menu application item class
var ApplicationMenuItem = new Lang.Class({
    Name: 'ApplicationMenuItem',
    Extends: PopupMenu.PopupBaseMenuItem,

    // Initialize menu item
    _init: function(button, app, area) {
        this.parent();
        this._app = app;
        this.app = app;
        this._button = button;

        this._area = area; //Add kylin code to mark where the app display
        //Add kylin code to user kylin type
        this.actor.remove_all_children();
        this.actor.add_style_class_name('app-menu-item');
        if(this._area == APP_IN_FIRST_AREA_BEGIN || this._area == APP_IN_FIRST_AREA_RECENT || this._area == APP_IN_SEARCH_AREA) {
            this._iconBin = new St.Bin({
                style_class: 'app-icon-dis',
                x_align: Clutter.ActorAlign.CENTER
            });
        }
        else {
            this._iconBin = new St.Bin({
                style_class: 'app-icon',
            });
        }
        this.actor.add_child(this._iconBin);

        let appLabel = new St.Label({
            text: app.get_name(),
            y_expand: true,
	    style_class: 'back-label', //Add kylin code
            y_align: Clutter.ActorAlign.CENTER
        });
        this.actor.add_child(appLabel);
        this.actor.label_actor = appLabel;

        let textureCache = St.TextureCache.get_default();
        let iconThemeChangedId = textureCache.connect('icon-theme-changed',
                                                      Lang.bind(this, this._updateIcon));
        this.actor.connect('destroy', Lang.bind(this, function() {
                textureCache.disconnect(iconThemeChangedId);
        }));
 
        this._updateIcon();
        this._menu = null;
        this._menuManager = new PopupMenu.PopupMenuManager(this);
        this._menuTimeoutId = 0;

        this._draggable = DND.makeDraggable(this.actor);
        this.isDraggableApp = true;
        this._draggable.connect('drag-begin', Lang.bind(this, this._onDragBegin));
        this._draggable.connect('drag-cancelled', Lang.bind(this, this._onDragCancelled));
        this._draggable.connect('drag-end', Lang.bind(this, this._onDragEnd));

        //Add kylin code for display tip text
        this.actor.connect('enter-event', Lang.bind(this, this._onMouseEnter));
        this.actor.connect('leave-event', Lang.bind(this, this._offMouseEnter));
    },

    _onDragBegin: function() {
        Main.overview.beginItemDrag(this);
    },

    _onDragCancelled: function() {
        Main.overview.cancelledItemDrag(this);
    },

    _onDragEnd: function() {
        Main.overview.endItemDrag(this);
    },

    _onKeyPressEvent: function (actor, event) {
        let symbol = event.get_key_symbol();
        if (symbol == Clutter.KEY_Return ||
            symbol == Clutter.KEY_KP_Enter) {
            this.activate(event);
            return Clutter.EVENT_STOP;
        }
        return Clutter.EVENT_PROPAGATE;
    },

    get_app_id: function() {
        return this._app.get_id();
    },

    //Add kylin code for get app desc
    get_app_description: function() {
        return this._app.get_description();
    },

    getDragActor: function() {
       return this._app.create_icon_texture(APPLICATION_ICON_SIZE);
    },

    // Returns the original actor that should align with the actor
    // we show as the item is being dragged.
    getDragActorSource: function() {
        return this.actor;
    },

    // Activate menu item (Launch application)
    activate: function(event) {
        this._addToRecents();//Add kylin code to add open apps to recent list
	this._button._reload();
        this._app.open_new_window(-1);
        this._button.menu.toggle();
//	this._button.tipLabelHiddle();
        this.parent(event);
    },

    // Set button as active, scroll to the button
    setActive: function(active, params) {
        if (active && !this.actor.hover)
            this._button.scrollToButton(this);

        this.parent(active, params);
    },

    //Add kylin code for mouse in
    _onMouseEnter: function(actor, event) {
//        let [x,y]= event.get_coords();
//        if(this.get_app_description() != null)
//            this._button.tipLabelShow(this.get_app_description(), x, y);
    },

    //Add kylin code for mouse out
    _offMouseEnter: function(actor, event) {
//        this._button.tipLabelHiddle();
    },

    setFakeActive: function(active) {
        if (active) {
            this._button.scrollToButton(this);
            //this.actor.add_style_pseudo_class('active');
            this.actor.add_style_class_name('selected');
        } else {
            //this.actor.remove_style_pseudo_class('active');
            this.actor.remove_style_class_name('selected');
        }
    },

    // Grab the key focus
    grabKeyFocus: function() {
        this.actor.grab_key_focus();
    },

    // Update the app icon in the menu
    _updateIcon: function() {
        this._iconBin.set_child(this._app.create_icon_texture(APPLICATION_ICON_SIZE));
    },

    //Add kylin code 
    _removeMenuTimeout: function() {
        if (this._menuTimeoutId > 0) {
            Mainloop.source_remove(this._menuTimeoutId);
            this._menuTimeoutId = 0;
        }
    },

    //Add kylin code 
    popupMenu: function(event) {
        this._removeMenuTimeout();

        if (this._draggable)
            this._draggable.fakeRelease();

        if (!this._menu) {
            this._menu = new SecondaryMenu.AppItemMenu(this, this._button, this._area);
            this._menu.connect('activate-window', Lang.bind(this, function (menu, window) {
                this.activateWindow(window);
            }));
            this._menu.connect('open-state-changed', Lang.bind(this, function (menu, isPoppedUp) {
                if (!isPoppedUp)
                    this._onMenuPoppedDown();
            }));
            let id = Main.overview.connect('hiding', Lang.bind(this, function () { this._menu.close(); }));
            this.actor.connect('destroy', function() {
                Main.overview.disconnect(id);
            });

            this._menuManager.addMenu(this._menu);
        }

        this.emit('menu-state-changed', true);

        //Add kylin code to set positon
        let settings = new Gio.Settings({ schema_id: DESKTOP_INTERFACE_SCHEMA });
        let factor = settings.get_double(KEY_TEXT_SCALING_FACTOR);
        let [stage_x, stage_y] = event.get_coords();
        stage_x = stage_x - 250 * factor;
        stage_y = 0;
        this._menu.actor.set_position(parseInt(stage_x), parseInt(stage_y));
        this.actor.set_hover(true);
        this._menu.popup();
        this._menuManager.ignoreRelease();

        return false;
    },

    //Add kylin code for add to recent list
    _addToRecents: function() {
        let canRecent = global.settings.is_writable(this._button.RECENT_APPS_KEY);
        if (canRecent) {
            let rens = this._button.appRecentRecordInstance;
            let isRencent = rens.isRecord(this.get_app_id());
            if(isRencent){
                rens.moveRecordToPos(this.get_app_id(), 0);
            } else {
                rens.addRecordAtPos(this.get_app_id(), 0);
           }
        }
    },

    //Add kylin code 
    _onButtonPressEvent: function(actor, event) {
        let button = event.get_button();
        if (button == 3) {
//            this._button.tipLabelHiddle();
            this.popupMenu(event);
            return Clutter.EVENT_STOP;
        }
        return Clutter.EVENT_PROPAGATE;
    },
 
    _onButtonReleaseEvent: function (actor, event) {
        let button = event.get_button();
        if (button == 1) {
            this.activate(event);
        }
        return Clutter.EVENT_STOP;
    },

    //Add kylin code 
    _onMenuPoppedDown: function() {
        this.actor.sync_hover();
        this.emit('menu-state-changed', false);
    }
});

// Menu Category item class
var CategoryMenuItem = new Lang.Class({
    Name: 'CategoryMenuItem',
    Extends: BaseMenuItem,

    // Initialize menu item
    _init: function(button, category) {
        this.parent();
        this._category = category;
        this._button = button;
	//Add kylin code to use kylin style
	this.actor.remove_all_children();
        this.actor.add_style_class_name('category-menu-item');

        let name;
        if (this._category) {
            name = this._category.get_name();
        } else {
            name = _("Favorites");
        }

        if(name == "Documentation")
            name = _("Documentation");

        this._icon = new St.Icon({
            icon_name: 'folder', //Add kylin code
            style_class: 'popup-menu-icon',
            icon_size: APPLICATION_ICON_SIZE
        });
        this.actor.add_child(this._icon);
        let categoryLabel = new St.Label({
            text: name,
	    style_class: 'back-label',
            y_expand: true,
            y_align: Clutter.ActorAlign.CENTER
        });
        this.actor.add_child(categoryLabel);
        this.actor.label_actor = categoryLabel;
    },

    // Activate menu item (Display applications in category)
    activate: function(event) {
        this._button.selectCategory(this._category);
	this._button.scrollToButton(this); //Add kylin code
        this.parent(event);
    },

    // Set button as active, scroll to the button
    setActive: function(active, params) {
        if (active && !this.actor.hover) {
            this._button.scrollToButton(this);
        }
        this.parent(active, params);
    }
});

// Place Info class
var PlaceInfo = new Lang.Class({
    Name: 'PlaceInfo',

    // Initialize place info
    _init: function(file, name, icon, default_dir) {
        this.file = file;
	this._default_dir = default_dir;
        this.name = name ? name : this._getFileName();
        this.icon = icon ? new Gio.ThemedIcon({ name: icon }) : this.getIcon();
    },

    // Launch place with appropriate application
    launch: function(timestamp) {
	let launchContext = global.create_app_launch_context(timestamp, -1);
	if(this._default_dir == GLib.UserDirectory.DIRECTORY_DOCUMENTS
	   || this._default_dir == GLib.UserDirectory.DIRECTORY_DOWNLOAD
	   || this._default_dir == GLib.UserDirectory.DIRECTORY_MUSIC 
	   || this._default_dir == GLib.UserDirectory.DIRECTORY_PICTURES
	   || this._default_dir == GLib.UserDirectory.DIRECTORY_VIDEOS)
	{
	    let i;
	    if(this._default_dir == GLib.UserDirectory.DIRECTORY_DOCUMENTS)
	    {
		i = 5;
	    }
	    else if(this._default_dir == GLib.UserDirectory.DIRECTORY_DOWNLOAD)
	    {
		i = 2; 
	    }
	    else if(this._default_dir == GLib.UserDirectory.DIRECTORY_MUSIC)
	    {
		i = 6; 
	    }
	    else if(this._default_dir == GLib.UserDirectory.DIRECTORY_PICTURES)
	    {
		i = 7; 
	    }
	    else
	    {
		i = 8;
	    }
	    let shell = "/usr/bin/open_palce.sh " + i;
	    print(shell);
 	    Util.spawnCommandLine(shell);
	}
	else
	{
            Gio.AppInfo.launch_default_for_uri(this.file.get_uri(), launchContext);
	}
    },

    // Get Icon for place
    getIcon: function() {
        try {
            let info = this.file.query_info('standard::symbolic-icon', 0, null);
	        return info.get_symbolic_icon();
        
        } catch(e) {
            if (e instanceof GioIOErrorEnum) {
                if (!this.file.is_native()) {
                    return new Gio.ThemedIcon({ name: 'folder-remote-symbolic' });
                } else {
                    return new Gio.ThemedIcon({ name: 'folder-symbolic' });
                }
            }
        }
    },

    // Get display name for place
    _getFileName: function() {
        try {
            let info = this.file.query_info('standard::display-name', 0, null);
            return info.get_display_name();
        } catch(e) {
            if (e instanceof Gio.IOErrorEnum) {
                return this.file.get_basename();
            }
        }
    }
});
Signals.addSignalMethods(PlaceInfo.prototype);

// Menu Place Shortcut item class
var PlaceMenuItem = new Lang.Class({
    Name: 'PlaceMenuItem',
    Extends: BaseMenuItem,

    // Initialize menu item
    _init: function(button, info, dir) {
        this.parent();
        this.actor.remove_all_children(); //Add kylin code to clear all children
        this.actor.add_style_class_name('place-menu-item'); //Add kylin code for use kylin style

        this._button = button;
        this._info = info;
	this._dir = dir; //Add kylin code
	let _name;
	//Add kylin code use icon
        this._icon = new St.Icon({
            style_class: 'user-icon'
        });

        //Add kylin code to get user name       
        if(dir == "home") {
            info.name = GLib.get_user_name();
        }

        this._label = new St.Label({ text: info.name, y_expand: true,
                                  style_class: 'place-menu-item-label',
                                  y_align: Clutter.ActorAlign.CENTER });

        this.actor.add_child(this._label);
        this._changedId = this._info.connect('changed',
                                       Lang.bind(this, this._propertiesChanged));
	this._setTipTextAndIcon(); //Add kylin code 
	this.actor.connect('enter-event', Lang.bind(this, this._onMouseEnter));
        this.actor.connect('leave-event', Lang.bind(this, this._offMouseEnter));
    },

    // Destroy menu item
    destroy: function() {
        if (this._changedId) {
            this._info.disconnect(this._changedId);
            this._changedId = 0;
        }
        this.parent();
    },

    // Activate (launch) the shortcut
    activate: function(event) {
	//Add kylin code
	if(this._dir == "computer"){
            Util.spawnCommandLine("nautilus computer:///");
        } else {
            this._info.launch(event.get_time());
        }
        this._button.user.swapIcon(this._button.user._userIcon);
//        this._button.tipLabelHiddle();
        this._button.menu.toggle();
        this.parent(event);
    },

    //Add kylin code for mouse in
    _onMouseEnter: function(actor, event) {
        this._button.user.swapIcon(this._icon);
//        let [x,y]= event.get_coords();
//        this._button.tipLabelShow(this._tipText, x, y);
    },

    //Add kylin code for mouse out
    _offMouseEnter: function(actor, event) {
        this._button.user.swapIcon(this._button.user._userIcon);
//        this._button.tipLabelHiddle();
    },

    //Add kylin code for set tip text and icon
    _setTipTextAndIcon: function() {
        let iconPath;
        this._tipText = "";
        if(this._dir == "home") {
            iconPath = home_icon_path;
            this._tipText = _("Open your personal folder.");
        }
        else if(this._dir == GLib.UserDirectory.DIRECTORY_DOCUMENTS){
            iconPath = documents_icon_path;
            this._tipText = _("Access letters,reports,notes,and other kinds of documents.");
        }
        else if(this._dir == GLib.UserDirectory.DIRECTORY_PICTURES){
            iconPath = pictures_icon_path;
            this._tipText = _("View and organize digital pictures.");
        }
        else if(this._dir == GLib.UserDirectory.DIRECTORY_MUSIC){
            iconPath = music_icon_path;
            this._tipText = _("Play music and other audio files.");
        }
        else if(this._dir == GLib.UserDirectory.DIRECTORY_VIDEOS){
            iconPath = videos_icon_path;
            this._tipText = _("Watch home movies and other digital videos.");
        }
        else if(this._dir == GLib.UserDirectory.DIRECTORY_DOWNLOAD){
            iconPath = download_icon_path;
            this._tipText = _("Find Internet downloads and links to favorite websites.");
        }
        else if(this._dir == "computer"){
            iconPath = computer_icon_path;
            this._tipText = _("See the disk drivers and other hardware connected to your computer.");
        }

        //set icon
        if (GLib.file_test(iconPath, GLib.FileTest.EXISTS)) {
            this._icon.set_gicon(Gio.icon_new_for_string(iconPath));
        }
    },

    // Handle changes in place info (redisplay new info)
    _propertiesChanged: function(info) {
        this._icon.gicon = info.icon;
        this._label.text = info.name;
    }
});

/**
 * This class represents a SearchBox.
 */
var SearchBox = new Lang.Class({
    Name: 'Class',

    _init: function() {
        this.actor = new St.BoxLayout({
            style_class: 'search-box search-box-padding'
        });
        this._stEntry = new St.Entry({
            name: 'search-entry',
            hint_text: _("Type to search…"),
            track_hover: true,
            can_focus: true
        });
        this._findIcon = new St.Icon({
            style_class: 'search-entry-icon',
            icon_name: 'edit-find-symbolic',
            icon_size: 16
        });
        this._clearIcon = new St.Icon({
            style_class: 'search-entry-icon',
            icon_name: 'edit-clear-symbolic',
            icon_size: 16
        });
        this._stEntry.set_primary_icon(this._findIcon);
        this.actor.add(this._stEntry, {
            expand: true,
            x_align: St.Align.START,
            y_align: St.Align.START
        });

        this._text = this._stEntry.get_clutter_text();
        this._textChangedId = this._text.connect('text-changed', Lang.bind(this, this._onTextChanged));
        this._keyPressId = this._text.connect('key-press-event', Lang.bind(this, this._onKeyPress));
        this._keyFocusInId = this._text.connect('key-focus-in', Lang.bind(this, this._onKeyFocusIn));
        this._searchIconClickedId = 0;
        this._inputHistory = [];
        this._maxInputHistory = 5;

        this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
    },

    _pushInput: function(searchString) {
        if (this._inputHistory.length == this._maxInputHistory) {
            this._inputHistory.shift();
        }
        this._inputHistory.push(searchString);
    },

    _lastInput: function() {
        if (this._inputHistory.length != 0) {
            return this._inputHistory[this._inputHistory.length-1];
        }
        return '';
    },

    _previousInput: function() {
        if (this._inputHistory.length > 1) {
            return this._inputHistory[this._inputHistory.length-2];
        }
        return '';
    },

    getText: function() {
        return this._stEntry.get_text();
    },

    setText: function(text) {
        this._stEntry.set_text(text);
    },

    // Grab the key focus
    grabKeyFocus: function() {
        this._stEntry.grab_key_focus();
    },

    hasKeyFocus: function() {
        return this._stEntry.contains(global.stage.get_key_focus());
    },

    // Clear the search box
    clear: function() {
        this._stEntry.set_text('');
        this._stEntry.grab_key_focus();
        this.emit('cleared');
    },

    isEmpty: function() {
        return this._stEntry.get_text() == '';
    },

    _isActivated: function() {
        return this._stEntry.get_text() != '';
    },

    _setClearIcon: function() {
       this._stEntry.set_secondary_icon(this._clearIcon);
        if (this._searchIconClickedId == 0) {
            this._searchIconClickedId = this._stEntry.connect('secondary-icon-clicked',
                Lang.bind(this, this.clear));
        }
    },

    _unsetClearIcon: function() {
       if (this._searchIconClickedId > 0) {
            this._stEntry.disconnect(this._searchIconClickedId);
        }
        this._searchIconClickedId = 0;
        this._stEntry.set_secondary_icon(null);
    },

    _onTextChanged: function(entryText) {
        let searchString = this._stEntry.get_text();
        this._pushInput(searchString);
        if (this._isActivated()) {
            this._setClearIcon();
        } else {
            this._unsetClearIcon();
            if (searchString == '' && this._previousInput() != '') {
                this.emit('cleared');
            }
        }
        this.emit('changed', searchString);
    },

    _onKeyPress: function(actor, event) {
        let symbol = event.get_key_symbol();
        if (symbol == Clutter.KEY_Return ||
            symbol == Clutter.KEY_KP_Enter) {
             if (!this.isEmpty()) {
                this.emit('activate');
            }
            return Clutter.EVENT_STOP;
        }
        this.emit('key-press-event', event);
        return Clutter.EVENT_PROPAGATE;
    },

    _onKeyFocusIn: function(actor) {
        this.emit('key-focus-in');
        return Clutter.EVENT_PROPAGATE;
    },

    _onDestroy: function() {
        if (this._textChangedId > 0) {
            this._text.disconnect(this._textChangedId);
            this._textChangedId = 0;
        }
        if (this._keyPressId > 0) {
            this._text.disconnect(this._keyPressId);
            this._keyPressId = 0;
        }
        if (this._keyFocusInId > 0) {
            this._text.disconnect(this._keyFocusInId);
            this._keyFocusInId = 0;
        }
    }
});
Signals.addSignalMethods(SearchBox.prototype);

/**
 * This class is responsible for the appearance of the menu button.
 */
var MenuButtonWidget = new Lang.Class({
    Name: 'Class',

    _init: function() {
        this.actor = new St.BoxLayout({
            style_class: 'panel-status-menu-box',
            pack_start: false
        });
        this._arrowIcon = PopupMenu.arrowIcon(St.Side.BOTTOM);
        this._icon = new St.Icon({
            icon_name: 'start-here-symbolic',
            style_class: 'popup-menu-icon'
        });
        this._label = new St.Label({
            text: _("Applications"),
            y_expand: true,
            y_align: Clutter.ActorAlign.CENTER
        });

        this.actor.add_child(this._icon);
        this.actor.add_child(this._label);
        this.actor.add_child(this._arrowIcon);
    },

    getPanelLabel: function() {
        return this._label;
    },

    getPanelIcon: function() {
        return this._icon;
    },

    showArrowIcon: function() {
        if (!this.actor.contains(this._arrowIcon)) {
            this.actor.add_child(this._arrowIcon);
        }
    },

    hideArrowIcon: function() {
        if (this.actor.contains(this._arrowIcon)) {
            this.actor.remove_child(this._arrowIcon);
        }
    },

    showPanelIcon: function() {
        if (!this.actor.contains(this._icon)) {
            this.actor.add_child(this._icon);
        }
    },

    hidePanelIcon: function() {
        if (this.actor.contains(this._icon)) {
            this.actor.remove_child(this._icon);
        }
    },

    showPanelText: function() {
        if (!this.actor.contains(this._label)) {
            this.actor.add_child(this._label);
        }
    },

    hidePanelText: function() {
        if (this.actor.contains(this._label)) {
            this.actor.remove_child(this._label);
        }
    }
});
